from django import forms
from django.utils.translation import ugettext_lazy as _
from datetime import date, time, datetime, timedelta
from django.core.mail import EmailMessage

from jumbo.forms.models import ModelForm
import jumbo.middleware.thread_local as tl
from jumbo.utils.functions import get_settings
from jumbo.forms import DateWidget, DateTimeWidget

from timereport.models import Employee, Transit, STATUS_CODES


class Transito(ModelForm):
    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out',)

    def __init__(self,  *args, **kwargs):
        super(Transito, self).__init__(*args, **kwargs)
        self.fields['employee'].widget.choices = self.get_employee()
        self.fields['date_time_in'].widget= DateTimeWidget()
        self.fields['date_time_out'].widget= DateTimeWidget()


    def get_employee(self):
        user = tl.get_current_user()

        query = Employee.active_objects.all()

        if user.has_perm("timereport.can_insert_all_manual_transit"):
            query = query
        else:
            query = query.filter(user=user)

        query = query.order_by('user__last_name')

        for i in query:
            yield (i.id, i.name)


class TransitForm(Transito):
    def save(self):
        self.instance.status = 21
        super(TransitForm, self).save()

class PermitForm(Transito):
    def __init__(self, *args, **kwargs):
        super(PermitForm, self).__init__(*args, **kwargs)
        self.fields['date_time_in'].label = _("Start permit")
        self.fields['date_time_out'].label = _("Stop permit")

    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out', 'note')

    def save(self):
        self.instance.status = 22
        super(PermitForm, self).save()
        self.instance.send_mail()

class VacationForm(Transito):
    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out',)

    def __init__(self, *args, **kwargs):
        super(VacationForm, self).__init__(*args, **kwargs)
        self.fields['date_time_in'] = forms.DateField(label=_("Vacation Date From"), widget= DateWidget())
        self.fields['date_time_out'] = forms.DateField(label=_("Vacation Date To"), widget= DateWidget())

    def clean_date_time_in(self):
        self.cleaned_data['date_time_in'] = datetime.combine(
                self.cleaned_data['date_time_in'],
                time().min
        )
        return self.cleaned_data['date_time_in']

    def clean_date_time_out(self):
        self.cleaned_data['date_time_out'] = datetime.combine(
                self.cleaned_data['date_time_out'],
                time(23, 59, 59)
        )
        return self.cleaned_data['date_time_out']

    def save(self):
        self.instance.status = 23
        super(VacationForm, self).save()
        self.instance.send_mail()

class IllnessForm(Transito):
    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out',)

    def __init__(self, *args, **kwargs):
        super(IllnessForm, self).__init__(*args, **kwargs)
        self.fields['date_time_in'] = forms.DateField(label=_("Illness Date From"), widget= DateWidget())
        self.fields['date_time_out'] = forms.DateField(label=_("Illness Date To"), widget= DateWidget())


    def clean_date_time_in(self):
        self.cleaned_data['date_time_in'] = datetime.combine(
                self.cleaned_data['date_time_in'],
                time().min
        )
        return self.cleaned_data['date_time_in']

    def clean_date_time_out(self):
        self.cleaned_data['date_time_out'] = datetime.combine(
                self.cleaned_data['date_time_out'],
                time(23, 59, 59)
        )
        return self.cleaned_data['date_time_out']

    def save(self):
        self.instance.status = 25
        super(IllnessForm, self).save()
        self.instance.send_mail()


class InjuryForm(Transito):
    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out',)

    def __init__(self, *args, **kwargs):
        super(InjuryForm, self).__init__(*args, **kwargs)
        self.fields['date_time_in'] = forms.DateField(label=_("Date From"), widget= DateWidget())
        self.fields['date_time_out'] = forms.DateField(label=_("Date To"), widget= DateWidget())

    def clean_date_time_in(self):
        self.cleaned_data['date_time_in'] = datetime.combine(
                self.cleaned_data['date_time_in'],
                time().min
        )
        return self.cleaned_data['date_time_in']

    def clean_date_time_out(self):
        self.cleaned_data['date_time_out'] = datetime.combine(
                self.cleaned_data['date_time_out'],
                time(23, 59, 59)
        )
        return self.cleaned_data['date_time_out']

    def save(self):
        self.instance.status = 26
        super(InjuryForm, self).save()
        self.send_mail()


class DownloadTransitForm(forms.Form):
    date_begin = forms.DateField(widget=DateWidget)
    date_end = forms.DateField(widget=DateWidget)
